<?php

namespace Controllers;

use Services\ProductsAFService as Service;

class ProductsAFController
{

    public $dataSetErrors = [];
    private $cdProduct;
    private $key;

    public function __construct(Service $service)
    {
        $this->service = $service;
    }

    public function saveProduct($productsToSave, $dataToSave)
    {
        foreach ($productsToSave as $product) {
            $this->service->setProduct($product, $dataToSave);
        }
    }

    public function validatesDataProduct($order, $cdSupplier)
    {
        $this->dataSetErrors = [];
        $data = self::getDataToValidate($order);
        $products = $data['products'];
        
        for($cont = 0; $cont < count($products); $cont ++) {
            $product = self::parseProductObjectToArray($products[$cont]);
            $this->cdProduct = $product['codcliente'];
            $validCode = self::validatesProductCode();
            if ($validCode) {
                self::validatesRepeatedProductCode($products);
                self::validatesProductToSupplier($cdSupplier);
                self::validatesProductValues($product);
                self::validatesBestPrice($product, $cdSupplier, $data['cdFilDest'], $data['orderDate']);
            }
        }

        return $this->dataSetErrors;
    }

    public function parseProductObjectToArray($object)
    {
        return get_object_vars($object);
    }

    /**
     * Get the public keys of products on $order and return array
     * this products.
     *
     */
    public function getProductArray($order)
    {
        return $order->itens->{'det-item'};
    }

    private function getDataToValidate($order)
    {
        return [
            'cdFilDest' => (string) $order->{'cod-local'},
            'orderDate' => (string) $order->{'data-pedido'},
            'products' => self::getProductArray($order),
        ];
    }

    /**
     * Check if current product exist on DB.
     *
     */
    private function validatesProductCode()
    {
        $product = $this->service->getProduct($this->cdProduct);
        if (empty($product)) {
            $cdProduct = $this->cdProduct;
            $error = ("{$this->cdProduct} - Código do produto não existe");
            $index = 'invalidCode';
            self::setErrorOnDataSet($error, $index, $cdProduct);
            
            return false;
        }
        
        return true;
    }

    /**
     * Check if product is repeated on list of products.
     *
     */
    private function validatesRepeatedProductCode($products)
    {
        $countProduct = 0;
        
        for($cont = 0; $cont < count($products); $cont ++) {
            $product = self::parseProductObjectToArray($products[$cont]);
            if($product['codcliente'] == $this->cdProduct)
                $countProduct++;
        };
        
        if ($countProduct > 1) {
            $cdProduct = $this->cdProduct;
            $error = 'Produto repetido na nota';
            $index = 'repeated';
            self::setErrorOnDataSet($error, $index, $cdProduct);
        }
    }

    /**
     * Check if product is linked to the suplier.
     *
     */
    private function validatesProductToSupplier($cdSupplier)
    {
        $productToSupplier = $this->service->getProductToSupplier($cdSupplier, $this->cdProduct);
        if (!$productToSupplier[0]['CDPRODCOMPL']) {
            $cdProduct = $this->cdProduct;
            $error = 'Produto não cadastrado para o fornecedor.';
            $index = 'noSupplier';
            self::setErrorOnDataSet($error, $index, $cdProduct);
        }

    }

    /**
     * Check if price and amount are valid values.
     *
     */
    private function validatesProductValues($product)
    {
        self::validatesPrice((float) $product['vlr-unitario']);
        self::validatesAmount((float) $product['quantidade']);
    }

    private function validatesPrice($priceProduct)
    {
        if ($priceProduct <= 0) {
            $cdProduct = $this->cdProduct;
            $error = 'Preço inválido.';
            $index = 'invalidPrice';
            self::setErrorOnDataSet($error, $index, $cdProduct);
        }

    }

    /**
     * Check if product amount is valid.
     *
     */
    private function validatesAmount($amountProduct)
    {
        if ($amountProduct <= 0) {
            $cdProduct = $this->cdProduct;
            $error = 'Quantidade inválida.';
            $index = 'invalidAmount';
            self::setErrorOnDataSet($error, $index, $cdProduct);
        }

    }

    /**
     * Check if price of product is the same of less price in DB.
     *
     */
    private function validatesBestPrice($product, $cdSupplier, $cdFilDest, $orderDate)
    {   
        $centralCmp = $this->service->getCentralCmp($cdFilDest);
        $bestPrice = $this->service->getBestPrice($cdSupplier, $cdFilDest, (string) $product['codcliente'], $orderDate, $centralCmp['CDFILCENTCMP']);
        if (empty($bestPrice)) {
            $cdProduct = $this->cdProduct;
            $error = 'Não existe melhor preço cadastrado.';
            $index = 'noBestPrice';
            self::setErrorOnDataSet($error, $index, $cdProduct);
        } else {
            self::checkBestPrice($bestPrice, $product);
        }

    }

    /**
     * Check if product price is the same on DB.
     *
     */
    private function checkBestPrice($bestPrice, $product)
    {
        $priceFilter = array_filter($bestPrice, function ($price) use ($product) {
            return $price['VRPRECUNIITM'] == (float) str_replace(',', '.', (string) $product["vlr-unitario"]);
        });
        if (empty($priceFilter)) {
            $cdProduct = $this->cdProduct;
            $error = 'Preço na nota diferente do melhor preço do sistema.';
            $index = 'invalidBestPrice';
            self::setErrorOnDataSet($error, $index, $cdProduct);
        }
    }

    /**
     * Set error on dataSet with params as a result of validations.
     *
     */
    private function setErrorOnDataSet($error, $index, $cdProduct)
    {
        $this->dataSetErrors[$cdProduct][$index] = $error;
    }

}
